@@ -12,7 +12,6 @@ from typing import (
     Iterable,
     List,
     Optional,
-    Tuple,
     TypedDict,
 )
 
@@ -50,6 +49,7 @@ class Error(TypedDict):
 @jsondaora
 class Results(TypedDict):
     duration: Optional[float]
+    rampup_time: Optional[int]
     requested_rps_per_node: Optional[float]
     realized_requests: Optional[float]
     realized_rps: Optional[float]
@@ -62,6 +62,7 @@ class Results(TypedDict):
 class CallerConfig(TypedDict):
     duration: int
     rps_per_node: int
+    rampup_time: Optional[int]
 
 
 class Caller(DictDaora):
@@ -79,6 +80,9 @@ class Caller(DictDaora):
         await data_source.delete(self._script_key)
 
         for call_str in calls:
+            if not call_str:
+                continue
+
             try:
                 calls_group = orjson.loads(call_str)
 
@@ -117,10 +121,13 @@ class Caller(DictDaora):
                     logger.warning(f'Invalid line {call_str}')
 
         data_source.close()
+        await data_source.wait_closed()
 
     async def set_script(self, script_content: str) -> None:
         data_source = await self.get_data_source()
         await data_source.set(self._script_key, script_content)
+        data_source.close()
+        await data_source.wait_closed()
 
     async def get_data_source(self) -> aioredis.Redis:
         return await aioredis.create_redis_pool(self.data_source_target)
@@ -146,7 +153,9 @@ class Caller(DictDaora):
 
         try:
             self._run_calls(
-                configuration['duration'], configuration['rps_per_node']
+                configuration['duration'],
+                configuration['rps_per_node'],
+                configuration['rampup_time'],
             )
         except Exception as error:
             logger.exception(error)
@@ -164,8 +173,13 @@ class Caller(DictDaora):
         )
         data_source.close()
 
-    def _run_calls(self, duration: int, rps_per_node: int) -> None:
+    def _run_calls(
+        self, duration: int, rps_per_node: int, rampup_time: Optional[int]
+    ) -> None:
         with requests.Session() as http_data_source:
+            if rampup_time is None:
+                rampup_time = 0
+
             running_id = str(uuid.uuid4())
             logger.info(f'Starting calls for {running_id}')
             data_source = self.get_sync_data_source()
@@ -174,8 +188,6 @@ class Caller(DictDaora):
             executor = ThreadPoolExecutor(max_workers=100)
             calls_start_time = time.time()
             futures = []
-            calls_count_checkpoint = 0
-            wait_checkpoint = calls_start_time
             last_wait_time = 0.1
             run_call_func = make_run_call_function(
                 http_data_source, responses_status_map, latencies,
@@ -206,25 +218,25 @@ class Caller(DictDaora):
 
                         inputs = orjson.loads(inputs)
 
-                    def call_inputs() -> None:
-                        nonlocal calls_count
-                        for input_ in inputs:
+                    for input_ in inputs:
+
+                        def call_inputs() -> None:
+                            nonlocal calls_count
                             logger.debug(f'Getting output for: {input_}')
                             run_call_func(input_)
                             calls_count += 1
 
-                    futures.append(executor.submit(call_inputs))
-                    (
-                        last_wait_time,
-                        wait_checkpoint,
-                        calls_count_checkpoint,
-                    ) = wait_to_call(
-                        wait_checkpoint=wait_checkpoint,
-                        calls_count_checkpoint=calls_count_checkpoint,
-                        last_wait_time=last_wait_time,
-                        current_calls_count=calls_count,
-                        rps=rps_per_node,
-                    )
+                        futures.append(executor.submit(call_inputs))
+                        last_wait_time = wait_to_call(
+                            last_wait_time=last_wait_time,
+                            current_calls_count=calls_count,
+                            rps=rps_per_node,
+                            rampup_time=rampup_time,
+                            start_time=calls_start_time,
+                        )
+
+                        if not should_running(calls_start_time, duration):
+                            break
 
                 except Exception as error_:
                     logger.exception(type(error_).__name__)
@@ -238,6 +250,7 @@ class Caller(DictDaora):
             realized_requests = sum(responses_status_map.values())
             results = make_results(
                 duration,
+                rampup_time,
                 rps_per_node,
                 realized_requests,
                 latencies,
@@ -272,6 +285,7 @@ class Caller(DictDaora):
 
         all_results = await data_source.hgetall(self._results_key)
         all_durations = []
+        all_rampups = []
         all_requested_rps_per_node = []
         all_realized_requests = []
         all_realized_rps = []
@@ -283,6 +297,7 @@ class Caller(DictDaora):
         for result in all_results.values():
             result = orjson.loads(result)
             all_durations.append(result['duration'])
+            all_rampups.append(result['rampup_time'])
             all_requested_rps_per_node.append(result['requested_rps_per_node'])
             all_realized_requests.append(result['realized_requests'])
             all_realized_rps.append(result['realized_rps'])
@@ -293,6 +308,7 @@ class Caller(DictDaora):
 
         duration_ = float(np.mean(all_durations))
         realized_requests = float(np.sum(all_realized_requests))
+        rampup_time = int(np.mean(all_rampups))
         results = Results(
             duration=duration_,
             requested_rps_per_node=float(np.mean(all_requested_rps_per_node)),
@@ -307,8 +323,10 @@ class Caller(DictDaora):
             error=None,
             nodes_quantity=len(all_results),
             errors_count=0,
+            rampup_time=rampup_time,
         )
         data_source.close()
+        await data_source.wait_closed()
         return results
 
     def stop(self) -> None:
@@ -346,41 +364,52 @@ def make_run_call_function(
 
 
 def wait_to_call(
-    wait_checkpoint: float,
-    calls_count_checkpoint: int,
     last_wait_time: float,
     current_calls_count: int,
-    rps: int,
-) -> Tuple[float, float, int]:
+    rps: float,
+    rampup_time: int,
+    start_time: float,
+) -> float:
     wait_time = last_wait_time
     now = time.time()
+    elapsed_time = now - start_time
 
-    if now > wait_checkpoint + 1:
-        current_rps = (current_calls_count - calls_count_checkpoint) * round(
-            now - wait_checkpoint
-        )
-        rps_diff_percent = 1 - (current_rps / rps)
-        wait_checkpoint = now
-        calls_count_checkpoint = current_calls_count
+    if rampup_time > 0:
+        rps = rps_for_rampup(elapsed_time, rampup_time, rps)
 
-        if not -1.01 <= rps_diff_percent <= 0.01:
-            wait_time -= last_wait_time * rps_diff_percent
+    if elapsed_time > 1:
+        current_rps = current_calls_count / elapsed_time
+    else:
+        current_rps = current_calls_count
 
-        wait_time = wait_time if wait_time >= 0 else 0
+    if current_rps > rps:
+        wait_time += last_wait_time * rps / current_rps
+    elif current_rps < rps - 1:
+        wait_time -= last_wait_time * current_rps / rps
 
-        logger.debug(
-            f'current_rps={current_rps}, '
-            f'rps_diff_percent={rps_diff_percent}, '
-            f'wait_time={wait_time}'
-        )
+    logger.debug(
+        f'rps={rps}, '
+        f'current_rps={current_rps}, '
+        f'current_calls_count={current_calls_count}, '
+        f'wait_time={wait_time:.10f}, '
+        f'elapsed_time={elapsed_time}'
+    )
 
     time.sleep(wait_time)
 
-    return wait_time, wait_checkpoint, calls_count_checkpoint
+    return wait_time
+
+
+def rps_for_rampup(elapsed_time: float, rampup_time: int, rps: float) -> float:
+    if elapsed_time < rampup_time:
+        return rps * elapsed_time / rampup_time
+
+    return rps
 
 
 def make_results(
     duration: int,
+    rampup_time: int,
     rps_per_node: int,
     realized_requests: float,
     latencies: Iterable[float],
@@ -390,6 +419,7 @@ def make_results(
 ) -> Results:
     return Results(
         duration=duration,
+        rampup_time=rampup_time,
         requested_rps_per_node=rps_per_node,
         realized_requests=realized_requests,
         realized_rps=realized_requests / duration,
