@@ -1,6 +1,6 @@
 from typing import Generator
 
-from chunli.caller import Call
+from chunli.call import Call
 
 
 global get_calls_block
